/*
 * This example is from the Tools.h++ manual, version 7
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 */
#include <iostream.h>
#include <fstream.h>
#include <rw/auditbuf.h>
#include <rw/pstream.h>
#include <rw/cstring.h>
void doCrc (unsigned char c, void* x) {
  *(unsigned char*)x ^= c;
}

int main() {
  { // just a block to control variable lifetime
    unsigned char check = '\0';

    // create an output stream
    ofstream                            op("crc.pst");

    // create an RWAuditStreamBuffer that will do CRC
    RWAuditStreamBuffer               crcb(op,doCrc,&check);

    // create an RWpostream to put the data through.
RWpostream                           p(&crcb);

    // now send some random stuff to the stream
    p << RWCString("The value of Tools.h++ is at least ");
    p << (int)4;
    p << RWCString(" times that of the next best library!\n") ;
    p << RWCString("Pi is about ") << (double)3.14159 << '.';

    // finally, save the sum on the stream itself.
p << (unsigned int)check; // alters check, _after_ saving it...

    // just for fun, print out some statistics:
    cout << "We just saved " << crcb 
		  << " bytes of data to the file." << endl;
    cout << "The checksum for those bytes was " <<check << endl;
} // end of block

  // now read the data back in, checking to see if it survived.
  unsigned char check = '\0';

  // create an instream
  ifstream                            ip("crc.pst");

  // create an RWAuditStreamBuffer that will do CRC
  RWAuditStreamBuffer               crcb(ip,doCrc,&check);

  // create an RWpistream to interpret the bytes
  RWpistream                           p(&crcb);

  RWCString first, mid1, mid2;
  int value;
  double pi;
  char pnc;
  unsigned int savedCRC;
  unsigned char matchCRC;
  // read in the data. Don\'t read the checksum yet!
  p >> first >> value >> mid1 >> mid2 >> pi >> pnc;
  // save the checksum
  matchCRC = check;
  // Now it is safe to alter the running checksum by reading in
  // the one saved in the file.
p >> savedCRC;

  if(savedCRC != matchCRC) {
    cout << "Checksum error. Saved CRC: " << savedCRC
         << " built CRC: " << matchCRC << dec << endl;
  }
  else {
    cout << "The message was: " << endl;
    cout << first << value << mid1 << mid2 << pi << pnc << endl;
  }
  // just for fun, print out some statistics:
  cout	<< "We just read " << crcb 
		<< " bytes of data from the file." << endl;
  cout	<< "The checksum was " << matchCRC << flush;
  cout	<< " and the saved checksum was " << savedCRC << endl;
return 0;
}
