/*
 * This example is from the Tools.h++ manual, version 7
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 */
//Instructions:  This example requires C++ Standard Library support.
//               Compile with your include path set to the C++ Standard Library
//               header files and to the Tools.h++ root directory.

#include <rw/tvdeque.h>
#include <rw/pstream.h>
#include <fstream.h>
#include <iostream.h>

class Simple
{
  inline friend RWvostream& 
  operator<< (RWvostream& stream, const Simple& object)
  { stream << object.data_; return stream; }

  inline friend RWvistream&
  operator>>(RWvistream& stream, Simple& object)
  { stream >> object.data_; return stream; }

  inline friend RWFile& 
  operator<< (RWFile& stream, const Simple& object)
  { stream << object.data_; return stream; }

  inline friend RWFile&
  operator>>(RWFile& stream, Simple& object)
  { stream >> object.data_; return stream; }

  char  data_;

public:

   friend ostream& operator<< (ostream &str, Simple& object);

   Simple(char data) : data_(data){}
   Simple() : data_('\0'){}

   bool  operator<(const Simple& other) const
   { return data_ < other.data_; }

   bool  operator==(const Simple& other) const
   { return data_ == other.data_; }

};

   ostream& operator<< (ostream &str, Simple& object)
   { str << object.data_; return str; } 


int main()
{
  RWTValDeque<Simple> deq;

  Simple  i('e');
  Simple  h('v');
  Simple  g('a');
  Simple  f('W');
  Simple  e('e');
  Simple  d('u');
  Simple  c('g');
  Simple  b('o');
  Simple  a('R');
  
  deq.pushFront(e);
  deq.pushFront(d);
  deq.pushFront(c);
  deq.pushFront(b);
  deq.pushFront(a);
  deq.pushBack(f);
  deq.pushBack(g);
  deq.pushBack(h);
  deq.pushBack(i);
  {
    ofstream          f("deque.dat");
    RWpostream        po(f);
    po << deq;
  }

  RWTValDeque<Simple> d2;
  {
    ifstream          f("deque.dat");
    RWpistream        pi(f);

    pi >> deq;
  }

  Simple rest;

  while(!d2.isEmpty())
  {
    rest = d2.popFront();
    cout << rest; 
  }
  
  cout << endl;

  return 0;
}
