/*
 * This example is from the Tools.h++ manual, version 7
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 */
#include <rw/collect.h>
#include <rw/rwfile.h>
#include <iostream.h>

class Tangle : public RWCollectable
{

public:

  RWDECLARE_COLLECTABLE(Tangle)

  Tangle* nextTangle;
  int     someData;

  Tangle(Tangle* t = 0, int dat = 0)
  { nextTangle=t; someData=dat; }

  virtual void saveGuts(RWvostream&) const{;} 
  virtual void restoreGuts(RWvistream&){;}
  virtual void saveGuts(RWFile&) const;
  virtual void restoreGuts(RWFile&);

};

void Tangle::saveGuts(RWFile& file) const
{
   RWCollectable::saveGuts(file);       // Save the base class

   file.Write(someData);                // Save internals

   file << nextTangle;                  // Save the next link
}

void Tangle::restoreGuts(RWFile& file)
{
  RWCollectable::restoreGuts(file);     // Restore the base class

  file.Read(someData);                  // Restore internals

  file >> nextTangle;                   // Restore the next link
}


// Prints out data in list 

void checkList(Tangle* p, const char* msg)
{
   cout << msg << ": ";

   while (p)
   {
     cout << p -> someData << "  "; 

     p = p -> nextTangle;
   }

   cout << endl << endl;
}

void freeList(Tangle* p)
{
   if(p)
   {
     freeList(p -> nextTangle);
     delete p;
   }
}


RWDEFINE_COLLECTABLE(Tangle, 0x777)

main()
{
   Tangle *head = NULL, *head2;

   for (int i=9; i >= 0; i--)
     head = new Tangle(head,i);

   checkList(head, "List persisted");  // Check the original list

   {
     RWFile file("junk.dat");
     file << head;
   }

   RWFile file2("junk.dat");
   file2 >> head2;

   checkList(head2, "List restored"); // Check the restored list

   // Now two lists are on the heap, both need to be
   // deallocated.

   freeList(head);
   freeList(head2);

   return 0;
}
