/***************************************************************************
 *
 * ep_inset.cc - template definitions for RW_PAssoc<StdColl,RWColl,DataTypes>
 * 
 * $Id: ep_inset.cc,v 7.23 1996/03/16 13:23:08 jims Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ep_inset.cc,v $
 * Revision 7.23  1996/03/16 13:23:08  jims
 * Modify remove mfs to return value_type instead of const_value
 *
 * Revision 7.22  1996/03/02 00:16:12  jims
 * compiler.h --> rw/compiler.h
 *
 * Revision 7.21  1996/02/18 04:54:06  kyle
 * Millenium adds support for compilers wanting StdLib files with extensions.
 *
 * Revision 7.20  1996/02/18 01:40:40  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.19  1996/02/07 02:42:30  jims
 * Standard library extensions now in directory rw/stdex
 *
 * Revision 7.18  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.17  1996/01/05 00:30:30  griswolf
 * Scopus #2160: merge val, ptr (sorted) set-operation algorithms.
 *
 * Revision 7.16  1996/01/04 22:42:28  griswolf
 * make use of locally designed Union algorithm.
 *
 * Revision 7.15  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.14  1995/09/25 11:10:27  jims
 * Remove hash versions of set functions to e[pv]_hash.cc
 *
 * Revision 7.13  1995/09/22  16:33:12  griswolf
 * Fix bug in removeAll(function).
 *
 * Revision 7.12  1995/09/21  00:22:35  griswolf
 * Add new hash-based set operations.
 *
 * Revision 7.11  1995/09/09  01:13:53  jims
 * New alg for removeAll(bool (*fn)(...),void* x)
 *
 * Revision 7.10  1995/09/05  21:58:16  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.9  1995/08/31  12:03:51  jims
 * next rev
 *
 * Revision 7.8  1995/08/21  03:34:49  jims
 * pas std iterators to equal_range to get around compiler problems
 *
 * Revision 7.7  1995/08/11  23:31:50  jims
 * Cast away const to enable delete of const T* in non-standard
 * implementations
 *
 * Revision 7.6  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/compiler.h"

#ifndef RW_HEADER_WITH_EXTENSION
#  include <algorithm>
#else
#  include <algorithm.hpp>
#endif  /* RW_HEADER_WITH_EXTENSION */

#include "rw/stdex/algo.h"

#define TEMPLATE   template <class StdColl, class RWColl, class T>
#define PSA        RW_PAssoc<StdColl,RWColl,T>

TEMPLATE
void PSA::clearAndDestroy()
{
  for(iterator i = begin(); i != end(); ++i)
    delete *i;  // cast away const for non-std impls
  clear();
}

TEMPLATE
PSA::const_value
PSA::find(const_value key) const
{
  void* ret = 0;
  iterator iter = std().find((T*)key);

  if(iter != end()) {
    ret = *iter;
  }
  return (const_value) ret;
}

TEMPLATE
PSA::size_type
PSA::removeAll(const_value key)
{
  size_type ret = entries();
  pair<iterator,iterator>
    rng = std().equal_range((T*)key);

  if(rng.first != end())
    std().erase(rng.first,rng.second);

  return ret - entries();
}

/*
 * NB: Assumes that erase does not invalidate iterators referring
 * to elements in front of the one being erased.
 */
TEMPLATE    
PSA::size_type
PSA::removeAll(bool (*fn)(PSA::const_value, void*),void* x)
{
  size_type    ret = entries();
  
  while (entries() && (*fn)(*begin(),x))
    std().erase(begin());

  iterator iter(begin());
  iterator iter2;

  while (iter != end()) {
    iter2 = iter;
    ++iter2;
    if (iter2 != end() && (*fn)((const_value)*iter2,x))
      std().erase(iter2);
    else
      ++iter;
  }

  return ret - entries();
}


TEMPLATE
PSA::value_type
PSA::remove(bool (*fn)(PSA::const_value,void*), void* x)
{
  const_value ret = 0;
  iterator hit = begin();
  for(/**/; hit != end(); ++hit)
    if((*fn)(*hit,x))
      break;
  if(hit != end()) {
    ret = *hit;
    std().erase(hit);
  }
  return (value_type)ret;
}

TEMPLATE
void PSA::Union(const StdColl& stdcoll)
{
  insert_iterator<StdColl>  putHere(std(),end());
// call the algorithm provided with Tools.h++
  set_union_inplace(begin(),end(),
                    stdcoll.begin(),stdcoll.end(),
                    putHere,std().key_comp());
} 


TEMPLATE
void PSA::intersection(const StdColl& stdcoll)
{
// call the algorithm provided with Tools.h++
  set_intersection_inplace(std(),begin(),end(),
                           stdcoll.begin(),stdcoll.end(),
                           std().key_comp());
}

TEMPLATE
void PSA::difference(const StdColl& stdcoll)
{
// call the algorithm provided with Tools.h++
  set_difference_inplace(std(),begin(),end(),
                         stdcoll.begin(),stdcoll.end(),std().key_comp());
}  

TEMPLATE
void PSA::symmetricDifference(const StdColl& stdcoll)
{
// call the algorithm provided with Tools.h++
  set_symmetric_difference_inplace(std(),begin(),end(),
                           stdcoll.begin(),stdcoll.end(),std().key_comp());
}

#undef TEMPLATE
#undef PSA
