/***************************************************************************
 *
 * ep_scntn.cc - template definitions for RW_PCntnr<StdColl,RWColl,DataTypes>
 *
 * $Id: ep_scntn.cc,v 7.19 1996/03/22 22:03:09 griswolf Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 *
 * $Log: ep_scntn.cc,v $
 * Revision 7.19  1996/03/22 22:03:09  griswolf
 * Scopus #2898: Make things "bool correct".
 *
 * Revision 7.18  1996/02/18 01:40:52  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.17  1996/02/13 12:14:17  jims
 * Small mods to the interface
 *
 * Revision 7.16  1996/01/30 08:09:18  jims
 * Add definitions for newly out-of-lined functions
 *
 * Revision 7.15  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.14  1996/01/09 22:26:03  kevinj
 * Simpler external template persistence.
 *
 * Revision 7.13  1995/12/15 10:24:35  jims
 * Use RWUseStoreTable and RWUseReadTable for isomorphism
 *
 * Revision 7.12  1995/12/14 18:01:21  jims
 * saveOn, restoreFrom now global instead of member functions
 *
 * Revision 7.11  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.10  1995/11/13 12:44:57  jims
 * Use rwnil instead of nilValue()
 * Add value_type version of find
 *
 * Revision 7.9  1995/10/27 16:08:21  jims
 * Pass value_type(NULL) to RWMakeNewPtr instead of newValue to avoid warning
 *
 * Revision 7.8  1995/09/05  21:58:22  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.7  1995/08/23  23:44:04  kevinj
 * Changed saveOn and restoreFrom to return bool.
 *
 * Revision 7.6  1995/08/16  20:25:42  jims
 * #undef TEMPLATE and PSC
 *
 * Revision 7.5  1995/08/16  18:05:02  kevinj
 * Changed RWPersistTable and RWRestoreTable add() to insert().
 *
 * Revision 7.4  1995/08/12  00:25:10  kevinj
 * Template persistence design 2.3
 *
 * Revision 7.3  1995/08/11  23:38:18  kevinj
 * restoreFrom and saveOn for RWFile.
 *
 * Revision 7.2  1995/08/10  23:42:46  kevinj
 * Moved inline functions out of line.
 *
 * Revision 7.2  1995/08/10  23:42:46  kevinj
 * Moved inline functions out of line.
 *
 * Revision 7.1  1995/08/10  21:47:26  kevinj
 * Moved to Revision 7.1.
 *
 * Revision 1.1  1995/08/10  21:33:22  kevinj
 * Initial revision
 *
 ****************************************************************************/

#include "rw/ep_scntn.h"

#define TEMPLATE template <class StdColl, class RWColl, class DataTypes>
#define PSC      RW_PCntnr<StdColl, RWColl, DataTypes>

TEMPLATE void
PSC::apply(void (*fn)(PSC::const_value,void*),void *x) const {
  const_iterator iter = begin();
  while(iter != end()) {
    (*fn)(*iter,x);
    ++iter;
  }
}

TEMPLATE bool
PSC::contains(bool(*fn)(PSC::const_value,void*),void* x) const {
  const_iterator iter = begin();
  while(iter != end()) {
    if((*fn)(*iter,x)) break;
    ++iter;
  }
  return iter != end();
}

TEMPLATE bool
PSC::contains(PSC::const_value key) const {
  const_iterator iter = begin();
  while(iter != end()) {
    if(deref_equal()(*iter,key)) break;
    ++iter;
  }
  return iter != end();
}

TEMPLATE PSC::value_type
PSC::find(PSC::const_value key) const
{
    const_iterator iter = begin();
    while(iter != end()) {
      if(deref_equal()(*iter,key)) break;
      ++iter;
    }
    if (iter == end()) return rwnil; else return *iter;
}

TEMPLATE  PSC::value_type
PSC::find(bool (*fn)(PSC::const_value,void*), void* x) const
{
    const_iterator iter = begin();
    while(iter != end()) {
      if((*fn)(*iter,x)) break;
      ++iter;
    }
    if (iter == end()) return rwnil; else return *iter;
}

TEMPLATE  PSC::value_type
PSC::find(bool (*fn)(PSC::value_type,void*), void* x) const
{
    const_iterator iter = begin();
    while(iter != end()) {
      if((*fn)(*iter,x)) break;
      ++iter;
    }
    if (iter == end()) return rwnil; else return *iter;
}

TEMPLATE
PSC::size_type
PSC::index(bool (*fn)(PSC::const_value, void*), void* x) const
{
  const_iterator i = begin();
  size_type idx = 0;
  for(/**/; i != end(); ++i) {
    if((*fn)(*i,x))
      return idx;
    ++idx;
  }
  return npos;
}

TEMPLATE
PSC::size_type
PSC::index(PSC::const_value t) const
{
  const_iterator i = begin();
  size_type idx = 0;
  for(/**/; i != end(); ++i) {
    if(deref_equal()(*i,t))
      return idx;
    ++idx;
  }
  return npos;
}

TEMPLATE
PSC::size_type
PSC::occurrencesOf(bool (*fn)(PSC::const_value,void*),void* x) const {
  size_type ret = 0;
  const_iterator iter = begin();
  while(iter != end()) {
    if((*fn)(*iter,x)) ++ret;
    ++iter;
  }
  return ret;
}

TEMPLATE
PSC::size_type
PSC::occurrencesOf(PSC::const_value key) const {
  size_type ret = 0;
  const_iterator iter = begin();
  while(iter != end()) {
    if(deref_equal()(*iter,key)) ++ret;
    ++iter;
  }
  return ret;
}

#undef TEMPLATE
#undef PSC

