#ifndef __RWEP_SEQ_H__
#define __RWEP_SEQ_H__

/***************************************************************************
 *
 * ep_seq.h - RW_PSeq<StdColl,RWColl,DataTypes>
 *   
 *     : pointer-based interface components for STL Sequences
 * 
 * $Id: ep_seq.h,v 7.19 1996/05/08 00:19:07 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ep_seq.h,v $
 * Revision 7.19  1996/05/08 00:19:07  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.18  1996/02/18 01:40:57  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.17  1996/02/13 12:14:17  jims
 * Small mods to the interface
 *
 * Revision 7.16  1996/02/07 03:10:04  jims
 * Remove DEVNOTE regarding question about insertAt
 *
 * Revision 7.15  1996/01/30 08:05:45  jims
 * Change order of const and non-const min and maxElement functions
 * to fix mysterious porting problem!
 *
 * Revision 7.14  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.13  1995/12/20 18:56:52  pearson
 * Perform bounds checking for removeAt
 *
 * Revision 7.12  1995/12/20 16:20:07  pearson
 * Add bounds checking for insertAt
 *
 * Revision 7.11  1995/12/12  00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.10  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.9  1995/11/13 12:42:01  jims
 * Add overloaded mfs that take function pointers with value_type args
 *
 * Revision 7.8  1995/11/13 08:26:16  jims
 * Add get() mf for Tools-6 compatibility
 *
 * Revision 7.7  1995/09/05 21:42:12  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.6  1995/08/01  22:51:59  jims
 * Qualify include file name with "rw/"
 *
 * Revision 7.5  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/ep_scntn.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class StdColl, class RWColl, class T>
class RWTExport RW_PSeq : public RW_PCntnr<StdColl,RWColl,T>
{
public:
  typedef StdColl::value_type        value_type;
  typedef StdColl::size_type         size_type;
  typedef StdColl::iterator          iterator;
  typedef StdColl::const_iterator    const_iterator;
  typedef StdColl::reference         reference;
  typedef StdColl::const_reference   const_reference;
  typedef const T*                   const_value;


///// public methods
  void  append(value_type item) { std().insert(end(), item); }

  void  clear() { std().erase(begin(), end()); }

// clearAndDestroy() should be overridden by the leaf class whenever possible
// since this version makes two passes through the container: delete first,
// then erase().
  void          clearAndDestroy();

  value_type    get() { return removeFirst(); }

  bool          insert(value_type item)
    { std().insert(end(), item); return true; }

  void          insertAt(size_type index, value_type item)
    {
      if(index > 0)
        boundsCheck(index-1);
      
      iterator i = begin();
      advance(i,index);
      std().insert(i,item);
    }
  
  reference       minElement()
    { RWPRECONDITION(entries() != 0);
      return *min_element(begin(),end(),deref_less() ); }

  reference       maxElement()
    { RWPRECONDITION(entries() != 0);
      return *max_element(begin(),end(),deref_less() ); }

  const_reference minElement() const
    { RWPRECONDITION(entries() != 0);
      return *min_element(begin(),end(),deref_less() ); }

  const_reference maxElement() const
    { RWPRECONDITION(entries() != 0);
      return *max_element(begin(),end(),deref_less() ); }

  void  prepend(value_type item) { std().insert(begin(),item); }

// both removeAll() methods should be overridden by the leaf class whenever
// possible since these versions iterate size()+number_of_matches AND 
// make size()-first_match_location data swaps
  size_type     removeAll(const T* key);

  size_type     removeAll(bool (*fn)(const T*, void*),void*);
  size_type     removeAll(bool (*fn)(T*, void*),void*);

  value_type    removeAt(size_type index)
    {
      boundsCheck(index);
      iterator i = begin();
      advance(i,index);
      value_type ret = *i;
      std().erase(i);
      return ret;
    }

  value_type    removeFirst() { value_type ret = *begin();
                                std().erase(begin()); return ret; }

  value_type    remove(bool (*fn)(const_value, void*),void*);
  value_type    remove(bool (*fn)(value_type, void*),void*);

  value_type    remove(const_value key);

  size_type     replaceAll(const_value oldVal, value_type newVal);
  size_type     replaceAll(bool (*fn)(const_value, void*), void* x,
                           value_type newVal);
  size_type     replaceAll(bool (*fn)(value_type, void*), void* x,
                           value_type newVal);

protected:
  RW_PSeq() {} // prevent direct use
  RW_PSeq(const RW_PSeq<StdColl,RWColl,T>&);
};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/ep_seq.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* __RWEP_SEQ_H__ */
