/***************************************************************************
 *
 * ep_srtsq.cc - template definitions for
 *               RW_PSrtSeq<StdColl,RWColl,Compare,DataTypes>
 * 
 * $Id: ep_srtsq.cc,v 7.19 1996/03/14 23:54:53 pearson Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ep_srtsq.cc,v $
 * Revision 7.19  1996/03/14 23:54:53  pearson
 * #2825 - Suppress warning on Borland 4.52
 *
 * Revision 7.18  1996/03/13 23:28:49  griswolf
 * Note: tpsrtdli.cc now has index(key) more efficient that this version.
 *
 * Revision 7.17  1996/02/19 11:38:54  jims
 * Add missing terminating double-quote
 *
 * Revision 7.16  1996/02/18 04:49:05  kyle
 * Millenium adds support for compilers wanting extensions for StdLib files.
 *
 * Revision 7.15  1996/02/18 01:41:00  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.14  1996/01/30 23:04:59  jims
 * Move functions with "while" statements out of line
 *
 * Revision 7.13  1996/01/30 22:39:51  jims
 * Qualify formal argument types with 'PSSS::' to avoid 'Multiple
 * declaration...' error from Borland 4.5
 *
 * Revision 7.12  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.11  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.10  1995/11/17 08:11:59  jims
 * ;modify remove() to return pointer instead of boolean for
 * Tools 6 compatibility
 *
 * Revision 7.9  1995/09/23 00:39:55  griswolf
 * Fix bug in method remove(key).
 *
 * Revision 7.8  1995/09/05  21:58:34  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.7  1995/08/21  03:36:14  jims
 * Use std iterators with lower_bound to get around compiler problems
 *
 * Revision 7.6  1995/08/02  00:04:36  jims
 * Qualify include file names with "rw/"
 *
 * Revision 7.5  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/epfunc.h"
#ifndef RW_HEADER_WITH_EXTENSION
#  include <algorithm>
#else
#  include <algorithm.hpp>
#endif  /* RW_HEADER_WITH_EXTENSION */

#define FrmlTArgs  class StdColl, class RWColl, class Compare, class T
#define ActTArgs   StdColl, RWColl, Compare, T
#define TEMPLATE   template<FrmlTArgs>
#define PSS        RW_PSrtSeq<ActTArgs>

TEMPLATE
PSS::const_value
PSS::find(const_value key) const
{
  const_iterator hit = lower_bound(begin(),end(),key,Compare());
  if(hit != end() && **hit == *key) {
    return *hit;
  }
  return 0;
}

TEMPLATE
bool PSS::contains(const_value key) const
{
  const_iterator hit = lower_bound(begin(),end(),key,Compare());
  if(hit == end())
    return false;
  return (**hit == *key);
}

// Not efficient on classes without random access iterators. Override.
TEMPLATE 
PSS::size_type 
PSS::index(const_value key) const
{
  const_iterator it = lower_bound(begin(),end(),key,Compare());
  if (it == end()) return npos;
  
  size_type ret = 0;
  distance(begin(),it,ret);
  return ret;
}

TEMPLATE
PSS::size_type
PSS::occurrencesOf(const_value key) const
{
  size_type ret = 0;
  const_iterator iter = lower_bound(begin(),end(),key,Compare());
  while(iter != end() && **iter == *key) {
    ++iter;
    ++ret;
  }
  return ret;
}

TEMPLATE 
PSS::size_type
PSS::removeAll(const_value key)
{
  iterator start = lower_bound(begin(),end(),key,Compare());
  if(start == end())
    return 0;
  
  size_type oldsize = std().size();
  iterator bound = start;
  ++bound;
  while(bound != end() && **bound == *key) {
    ++bound;
  }
  std().erase(start,bound);
  return oldsize - std().size();
}

TEMPLATE
PSS::value_type PSS::remove(const_value key)
{
  iterator hit = lower_bound(begin(),end(),key,Compare());
  if(hit != end() && **hit == *key){
    value_type ret = *hit;
    std().erase(hit);
    return ret;
  }
  return rwnil;
}    

/** should be protected by #if RWDEBUG but xlC bug prevents it. **/
TEMPLATE
bool PSS::isSorted() const
{
  if(entries() < 2) return true; // empty and single are sorted
  const_iterator trail;
  for(const_iterator it = begin(); it != end();/**/)
  {
    trail = it++;
    if (it == end()) break;
    if (Compare()(*it,*trail))
      return false;
  }
  return true;
}

TEMPLATE
PSS::size_type
PSS::insert(const StdColl& coll)
{
  size_type ret = coll.size();
  if (ret) {
    iterator it = begin();
    const_iterator first = coll.begin();
    const_iterator last = coll.end();
    while(first != last)
      it = insertSorted(it,*first++);
  }
  return ret;
}

// protected functions

TEMPLATE
PSS::iterator
PSS::insertSorted(PSS::iterator hint, PSS::value_type a) 
{
  iterator hit;
  iterator last  = end();
  iterator first = begin();
  
  Compare comp = Compare();  // avoid warning "comp used before value set"
  if(hint == last)
    hit = upper_bound(first,last,a,comp);
  else {
    // test for "held is 'less than' a or a is not 'less than' held"
    // which amounts to "less than or equal"
    if(comp(*hint,a) || ! comp(a,*hint))
      hit = upper_bound(hint,last,a,comp);
    else // a must be "greater than" the contained value
      hit = upper_bound(first,hint,a,comp);
  }
  return std().insert(hit,a);
}

#undef FrmlTArgs
#undef ActTArgs
#undef TEMPLATE
#undef PSS
