#ifndef __RWEP_SRTSQ_H__
#define __RWEP_SRTSQ_H__

/***************************************************************************
 *
 * ep_srtsq.h - RW_PSrtSeq<StdColl,RWColl,Compare,DataTypes>
 *   
 * : pointer-based sorted-collection interface components for STL Sequences
 *       
 * 
 * $Id: ep_srtsq.h,v 7.15 1996/05/08 00:19:18 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ep_srtsq.h,v $
 * Revision 7.15  1996/05/08 00:19:18  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.14  1996/02/18 01:41:01  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.13  1996/01/30 23:04:59  jims
 * Move functions with "while" statements out of line
 *
 * Revision 7.12  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.11  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.10  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.9  1995/11/17 08:11:59  jims
 * modify remove() to return pointer instead of boolean for
 * Tools 6 compatibility
 *
 * Revision 7.8  1995/09/05 21:42:16  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.7  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 *
 ****************************************************************************/

#include "rw/ep_seq.h"
#include <algorithm>

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class StdColl, class RWColl, class Compare, class T>
class RWTExport RW_PSrtSeq : public RW_PSeq<StdColl,RWColl,T> {
private:
  typedef  RW_PSeq<StdColl,RWColl,T> base_type;

public:
  typedef StdColl::size_type            size_type;
  typedef base_type::value_type         value_type;
  typedef base_type::const_value        const_value;
  typedef base_type::reference          reference;
  typedef base_type::const_reference    const_reference;
  typedef base_type::iterator           iterator;
  typedef base_type::const_iterator     const_iterator;
  typedef base_type::deref_equal        deref_equal; 
  typedef base_type::deref_less         deref_less;

  bool          contains(const_value key) const;
  bool          contains(bool(*fn)(const_value,void*),void* x) const
    { return base_type::contains(fn,x); }

  const_value   find(const_value key) const;
  const_value   find(bool (*fn)(const_value,void*), void* x) const
    { return base_type::find(fn,x); }

  size_type     index(const_value key) const;
  size_type     index(bool (*fn)(const_value, void*), void* x) const
    { return base_type::index(fn,x); }
    
  size_type     insert(const StdColl& coll);
  bool          insert(value_type a)
    { insertSorted(begin(),a); return true; } // only fail thru exception

  size_type     occurrencesOf(const_value key) const;
  size_type     occurrencesOf(bool (*fn)(const_value,void*),void* x) const
    { return base_type::occurrencesOf(fn,x); }

// both removeAll() methods should be hidden by the leaf class whenever
// possible since these versions both iterate to find, AND iterate to remove. 
  size_type     removeAll(const_value key);
  size_type     removeAll(bool (*fn)(const_value, void*),void* x)
    { return base_type::removeAll(fn,x); }

  value_type    remove(const_value key);
  value_type    remove(bool (*fn)(const_value,void*),void* x)
    { return base_type::remove(fn,x); }

/** should be protected by #if RWDEBUG but xlC bug prevents it. **/
  bool          isSorted() const;
  
protected:
  RW_PSrtSeq() {} // prevent direct usage

  iterator insertSorted(iterator, value_type);
};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/ep_srtsq.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* __RWEP_SRTSQ_H__ */
