/***************************************************************************
 *
 * ev_inset.cc - template definitions for RW_VAssoc<StdColl,RWColl>
 * 
 * $Id: ev_inset.cc,v 7.13 1996/02/18 01:41:22 griswolf Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ev_inset.cc,v $
 * Revision 7.13  1996/02/18 01:41:22  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.12  1996/02/07 02:42:30  jims
 * Standard library extensions now in directory rw/stdex
 *
 * Revision 7.11  1996/01/05 00:30:07  griswolf
 * Scopus #2160: merge val, ptr (sorted) set-operation algorithms.
 *
 * Revision 7.10  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.9  1995/09/25 11:10:27  jims
 * Remove hash versions of set functions to e[pv]_hash.cc
 *
 * Revision 7.8  1995/09/21  00:22:35  griswolf
 * Add new hash-based set operations.
 *
 * Revision 7.7  1995/09/09  01:15:07  jims
 * New alg for removeAll(bool (*fn)(...),void* x)
 *
 * Revision 7.6  1995/09/05  21:59:22  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.5  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/
#include "rw/stdex/algo.h"

#define TEMPLATE template <class StdColl, class RWColl>
#define VSA      RW_VAssoc<StdColl, RWColl>

TEMPLATE
bool VSA::find(const_reference key, VSA::value_type& ret) const
{
  iterator iter = std().find(key);
  if(iter != end()) {
    ret = *iter;
    return true;
  }
  return false;
}

TEMPLATE
VSA::size_type  
VSA::removeAll(const_reference key)
{
  size_type ret = std().size();
  pair<iterator,iterator> rng = std().equal_range(key);
  if(rng.first != end())
    std().erase(rng.first,rng.second);
  return ret - std().size();
}

TEMPLATE
VSA::size_type  
VSA::removeAll(bool (*fn)(VSA::const_reference, void*), void* x)
{
  size_type    ret = entries();
  
  while (entries() && (*fn)(*begin(),x))
    std().erase(begin());

  iterator iter(begin());
  iterator iter2;

  while (iter != end()) {
    iter2 = iter;
    ++iter2;
    if (iter2 != end() && (*fn)(*iter2,x))
      std().erase(iter2);
    else
      ++iter;
  }

  return ret - entries();
}


TEMPLATE
bool VSA::remove(bool (*fn)(VSA::const_reference,void*), void* x)
{
  iterator hit = begin();
  for(/**/; hit != end(); ++hit)
    if((*fn)(*hit,x))
      break;
  if(hit != end()) {
    std().erase(hit);
    return true;
  }
  return false;
}

TEMPLATE
void VSA::Union(const StdColl& stdcoll)
{
// use algorithm supplied with Tools.h++
  insert_iterator<StdColl> putHere(std(),end());
  set_union_inplace(begin(),end(),stdcoll.begin(),stdcoll.end(),
                    putHere,std().key_comp());
} 


TEMPLATE
void VSA::intersection(const StdColl& stdcoll)
{
// use algorithm supplied with Tools.h++
  set_intersection_inplace(std(),begin(),end(),
                           stdcoll.begin(),stdcoll.end(),std().key_comp());
}


TEMPLATE
void VSA::difference(const StdColl& stdcoll)
{
// use algorithm supplied with Tools.h++
  set_difference_inplace(std(),begin(),end(),
                         stdcoll.begin(),stdcoll.end(),std().key_comp());
}


TEMPLATE
void VSA::symmetricDifference(const StdColl& stdcoll)
{
// use algorithm supplied with Tools.h++
  set_symmetric_difference_inplace(std(),begin(),end(),
                           stdcoll.begin(),stdcoll.end(),std().key_comp());
}  

#undef TEMPLATE
#undef VSA
