#ifndef __RWEV_INSET_H__
#define __RWEV_INSET_H__

/***************************************************************************
 *
 * ev_inset.h - RW_VAssoc<StdColl,RWColl>
 *   
 *     : value-based interface components for STL Associative Containers
 * 
 * $Id: ev_inset.h,v 7.20 1996/05/07 23:45:37 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ev_inset.h,v $
 * Revision 7.20  1996/05/07 23:45:37  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.19  1996/03/22 22:03:31  griswolf
 * Scopus #2898: Make things "bool correct".
 *
 * Revision 7.18  1996/02/18 01:41:24  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.17  1996/02/01 21:50:02  jims
 * Separate out hash-only set operations into separate header
 *
 * Revision 7.16  1996/01/04 16:27:21  pearson
 * Change isSubset to isSubsetOf, isProperSubset to isProperSubsetOf and
 * add multiset methods
 *
 * Revision 7.15  1995/12/15 00:29:11  pearson
 * Remove isUnequal and rename isEqual to isEquivalent
 * Move isEquivalent, isSubset and isProperSubset to the base class
 * Add access adjustment due to private inheritance
 *
 * Revision 7.14  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.13  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.12  1995/09/21 00:22:35  griswolf
 * Add new hash-based set operations.
 *
 * Revision 7.11  1995/09/05  21:53:16  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.10  1995/08/02  00:04:36  jims
 * Qualify include file names with "rw/"
 *
 * Revision 7.9  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 *
 ****************************************************************************/

#include "rw/ev_scntn.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class StdColl, class RWColl>
class RWTExport RW_VAssoc : public RW_VCntnr<StdColl, RWColl> {

private:
  typedef RW_VCntnr<StdColl, RWColl> base_type;

public:
  // Possible hack, need to redefine this typedef to avoid qualification
  //   for value_type& (that is w/o this you have to say StdColl::value_type&)
  typedef StdColl::value_type value_type;

// additional interface components
  void  clear() { std().erase(begin(),end()); }

  bool          remove(const_reference key)
  {
    iterator hit = std().find(key);
    if(hit != end()) {
      std().erase(hit);
      return true;
    }
    return false;
  }
  
  bool          remove(/** extends the interface **/
                          bool (*fn)(const_reference,void*),void*
                         );

  size_type     removeAll(const_reference key);
  size_type     removeAll(bool (*fn)(const_reference, void*),void*);

  void     Union(const StdColl&);
  void     Union(const RWColl& rwcoll)        { Union(rwcoll.std()); }

  void     intersection(const StdColl&);
  void     intersection(const RWColl& rwcoll) { intersection(rwcoll.std()); }

  void     difference(const StdColl&);
  void     difference(const RWColl& rwcoll)   { difference(rwcoll.std()); }

  void     symmetricDifference(const StdColl&);
  void     symmetricDifference(const RWColl& rwcoll)
                                { symmetricDifference(rwcoll.std()); }

  bool isSubsetOf(const RWColl& rhs) const
  {
     if(entries() > rhs.entries())
        return FALSE;

     const_iterator itr = begin();
     while(itr != end() && rhs.contains(*itr) )
          ++itr;

     return end() == itr;
  }


  bool isSubsetMultiOf(const RWColl& rhs) const
  {
     if(entries() > rhs.entries())
        return FALSE;

     const_iterator itr = begin();
     while(itr != end() && std().count(*itr) <=
                       rhs.std().count(*itr) )
         ++itr;
     return end() == itr;
  }

  bool isProperSubsetOf(const RWColl& rhs) const
  { return isSubsetOf(rhs) && (entries() < rhs.entries()); }

  bool isProperSubsetMultiOf(const RWColl& rhs) const
  { return isSubsetMultiOf(rhs) && (entries() < rhs.entries()); }


  bool isEquivalent(const RWColl& rhs) const
  { return isSubsetOf(rhs) && (entries() == rhs.entries()); }

  bool isEquivalentMulti(const RWColl& rhs) const
  { return isSubsetMultiOf(rhs) && (entries() == rhs.entries()); }


// overridden interface components
  bool  contains(const_reference key) const
  {
    iterator iter = std().find(key);
    return iter != end();
  }
  bool find(const_reference, value_type& ret) const;
  
  
  size_type occurrencesOf(const_reference key) { return std().count(key); }
  
// Avoid hiding overloads
  bool  contains(bool(*fn)(const_reference,void*), void* x) const
    { return base_type::contains(fn,x); }

  bool  find(bool(*fn)(const_reference,void*), void* x, value_type& hit) const
    { return base_type::find(fn, x, hit); }

  size_type occurrencesOf(bool(*fn)(const_reference,void*), void* x) const
    { return base_type::occurrencesOf(fn, x); }
};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/ev_inset.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* __RWEV_INSET_H__ */
