#ifndef __RWEV_SCNTN_H__
#define __RWEV_SCNTN_H__

/***************************************************************************
 *
 * ev_scntn.h - RW_VCntnr<StdColl,RWColl>
 *
 *     : value-based interface components for STL Containers
 *
 * $Id: ev_scntn.h,v 7.22 1996/05/07 23:45:59 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 *
 * $Log: ev_scntn.h,v $
 * Revision 7.22  1996/05/07 23:45:59  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.21  1996/03/14 00:41:18  griswolf
 * Scopus #2555: Remove devnote (did not change code).
 *
 * Revision 7.20  1996/02/18 01:41:32  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.19  1995/12/20 23:19:20  kevinj
 * External template persistence using RW*Persistor.
 *
 * Revision 7.18  1995/12/15 10:22:52  jims
 * Bring in min_element and max_element mfs from ev_cntnr.h
 * Work around 'no default args in function templates' problem
 *
 * Revision 7.17  1995/12/14 17:54:25  jims
 * saveOn, restoreFrom now global instead of member functions
 *
 * Revision 7.16  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.15  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.14  1995/09/05 21:53:25  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.13  1995/08/23  22:42:27  kevinj
 * Changed saveOn and restoreFrom to return bool.
 *
 * Revision 7.12  1995/08/17  22:55:28  kevinj
 * Added RW_DEFINE_PERSISTABLE_COLLECTION.
 *
 * Revision 7.11  1995/08/14  05:58:37  jims
 * include edefs.h, forward declare RWFile, RWv[io]stream
 *
 * Revision 7.10  1995/08/12  00:25:10  kevinj
 * Template persistence design 2.3
 *
 * Revision 7.9  1995/08/11  19:57:32  kevinj
 * First pass at RWFile persistence.
 *
 * Revision 7.9  1995/08/11  19:57:32  kevinj
 * First pass at RWFile persistence.
 *
 * Revision 7.8  1995/08/11  17:03:34  kevinj
 * Template persistence 2.2.
 *
 * Revision 7.7  1995/08/10  21:36:08  kevinj
 * Moved out of line functions to *.cc.
 *
 * Revision 7.7  1995/08/10  21:36:08  kevinj
 * Moved out of line functions to *.cc.
 *
 * Revision 7.6  1995/07/28  17:53:35  kevinj
 * Modified in accordance with Template Persistence Design Document 2.1.
 *
 * Revision 7.5  1995/07/27  23:14:34  kevinj
 * Added heading.
 *
 ****************************************************************************/



#include "rw/edefs.h"
#include "rw/ev_cntnr.h"

class RWExport RWFile;
class RWExport RWvistream;
class RWExport RWvostream;

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class StdColl, class RWColl>
class RWTExport RW_VCntnr : public RW_VBase<StdColl, RWColl>
{
public:
  typedef StdColl::value_type value_type;
  typedef StdColl::size_type size_type;
  typedef StdColl::const_reference          const_reference;

  bool          find(
            const_reference,
            value_type& val
                     ) const;

  bool          find(
            bool(*fn)(const_reference,void*),
            void* x, value_type& val
                      ) const;

  const_reference minElement() const
    { RWPRECONDITION(!isEmpty()); return *min_element(begin(),end()); }

  const_reference maxElement() const
  { RWPRECONDITION(!isEmpty()); return *max_element(begin(),end()); }

  reference       minElement()
  {
    RWPRECONDITION(!isEmpty());
#   ifdef _MSC_VER
      return (reference) *min_element(begin(),end());
#   else
      return (reference) *(iterator)min_element(begin(),end());
#   endif  
  }

  reference       maxElement()
  {
    RWPRECONDITION(!isEmpty());
#   ifdef _MSC_VER
      return (reference) *max_element(begin(),end());
#   else
      return (reference) *(iterator)max_element(begin(),end());
#   endif
  }

protected:
  RW_VCntnr() { } // prevent direct usage
  RW_VCntnr(const RW_VCntnr<StdColl,RWColl>&) { }
};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/ev_scntn.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* __RWEV_SCNTN_H__ */
