#ifndef __RWEV_SEQ_H__
#define __RWEV_SEQ_H__

/***************************************************************************
 *
 * ev_seq.h - RW_VSeq<StdColl,RWColl>
 *   
 *     : value-based interface components for STL Sequences
 * 
 * $Id: ev_seq.h,v 7.20 1996/05/07 23:46:08 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ev_seq.h,v $
 * Revision 7.20  1996/05/07 23:46:08  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.19  1996/02/18 01:41:38  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.18  1996/01/30 21:49:03  jims
 * Take functions out of line to avoid Borland warnings
 *
 * Revision 7.17  1996/01/02 16:11:58  pearson
 * Use boundsCheck for removeAt to conform semantics to non-STL version.
 *
 * Revision 7.16  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.15  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.14  1995/11/13 04:56:53  jims
 * Add get() member function for source compatiblilty with Tools-6
 * templates
 *
 * Revision 7.13  1995/09/18 19:07:07  pearson
 * Fixed logic in RWPRECONDITION statement
 *
 * Revision 7.12  1995/09/12  16:46:41  pearson
 * Fixed typo
 *
 * Revision 7.11  1995/09/05  21:53:31  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.10  1995/08/30  23:30:22  jims
 * Fix n_dbg typo in removeAt
 *
 * Revision 7.9  1995/08/01  21:37:34  jims
 * Qualify include file with "rw/"
 *
 * Revision 7.8  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/ev_scntn.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class StdColl, class RWColl>
class RWTExport RW_VSeq : public RW_VCntnr<StdColl, RWColl>
{
public:
  typedef StdColl::value_type        value_type;
  typedef StdColl::size_type         size_type;
  typedef StdColl::iterator          iterator;
  typedef StdColl::const_iterator    const_iterator;
  typedef StdColl::reference         reference;
  typedef StdColl::const_reference   const_reference;
  typedef const value_type&          const_val_ref;

  void          append(const_reference item)
    { std().insert(end(),item); }

  void          clear() { std().erase(begin(), end()); }

  bool  insert(const_reference key)
    { std().insert(end(),key); return true; } // fail only thru exception

  value_type    get() { return removeFirst(); }
  
  void          insertAt(size_type index, const_reference item)
    {
      // index equal to number of entries OK (inserts at end)
      if(index > 0) boundsCheck(index-1);
      iterator i = begin();
      advance(i,index);
      std().insert(i,item);
    }
  
  void          prepend(const_reference item)
    { std().insert(begin(),item); }

  bool          remove(bool (*fn)(const_reference,void*),void*);
  bool          remove(const_reference key);

  size_type     removeAll(const_reference key);
  size_type     removeAll(bool (*fn)(const_reference, void*),void*);

  value_type    removeAt(size_type index);

  value_type    removeFirst();

  size_type     replaceAll(const value_type& oldVal, const value_type& newVal);
  size_type     replaceAll(bool (*fn)(const value_type&, void*), void* x,
                           const value_type& newVal);

// hide some things
private:
  RW_VSeq(const RW_VSeq<StdColl,RWColl>&);

protected:
  RW_VSeq() {} // prevent direct use

};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/ev_seq.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* __RWEV_SEQ_H__ */
