#ifndef __RWEV_SRTSQ_H__
#define __RWEV_SRTSQ_H__

/***************************************************************************
 *
 * ev_srtsq.h - RW_VSrtSeq<StdColl,RWColl,Compare>
 *   
 * : value-based sorted-collection interface components for STL Sequences
 * 
 * $Id: ev_srtsq.h,v 7.14 1996/05/07 23:46:16 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: ev_srtsq.h,v $
 * Revision 7.14  1996/05/07 23:46:16  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.13  1996/03/15 00:46:10  griswolf
 * Scopus #2555: Make index more efficient.
 *
 * Revision 7.12  1996/02/18 01:41:41  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.11  1996/01/30 23:04:59  jims
 * Move functions with "while" statements out of line
 *
 * Revision 7.10  1996/01/30 08:08:17  jims
 * Remove use of obsolete std_iterator type
 *
 * Revision 7.9  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.8  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.7  1995/09/05 21:53:36  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.6  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/ev_seq.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class StdColl, class RWColl, class Compare>
class RWTExport RW_VSrtSeq : public RW_VSeq<StdColl,RWColl>
{
public:
  typedef StdColl::const_reference const_reference;
  typedef bool (*testFunction)(const_reference, void*);
  typedef RW_VSeq<StdColl,RWColl> base_type;
  typedef StdColl::const_iterator iterator;

  bool          contains(const_reference) const;
  bool          contains(bool(*fn)(const_reference,void*),void* x) const
    { return base_type::contains(fn,x); }

  bool          find(const_reference, reference) const;
  bool          find(bool(*fn)(const_reference,void*),void* x,
                     reference hit) const
    { return base_type::find(fn,x,hit); }

  size_type     index(const_reference key) const;
  size_type     index(bool (*fn)(const_reference, void*), void* x) const
    { return base_type::index(fn,x); }

  size_type     insert(const StdColl& coll);
  bool          insert(const_reference a)
    { insertSorted(begin(),a); return true; } // only fail thru exception

  size_type     occurrencesOf(const_reference) const;
  size_type     occurrencesOf(bool (*fn)(const_reference,void*),
                              void* x) const
    { return base_type::occurrencesOf(fn,x); }

  size_type     removeAll(const_reference);
  size_type     removeAll(bool (*fn)(const_reference, void*),void* x)
    { return base_type::removeAll(fn,x); }
  
  bool          remove(const_reference key);
  bool          remove(bool (*fn)(const_reference,void*),void* x)
    { return base_type::remove(fn,x); }

/** should be guarded by #if RWDEBUG but xlC bug prevents it. **/
  bool          isSorted() const;
  
protected:
  iterator insertSorted(iterator, const_reference);

protected:
  RW_VSrtSeq() { }  // can only be used as base class
  RW_VSrtSeq(const RW_VSrtSeq<StdColl,RWColl,Compare>&);
};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/ev_srtsq.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* __RWEV_SRTSQ_H__ */
