#ifndef __RWTPHSET_H__
#define __RWTPHSET_H__

/***************************************************************************
 *
 * tphset.h - RWTPtrHashSet<T,H,EQ>
 *   
 *     : pointer-based set wrapper for STL-like rw_hashset
 * 
 * $Id: tphset.h,v 7.25 1996/05/07 20:45:18 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: tphset.h,v $
 * Revision 7.25  1996/05/07 20:45:18  kyle
 * Added export pragma for Mac code fragments
 * Updated base_type for RW_BROKEN_ACCESS_ADJUST
 *
 * Revision 7.24  1996/02/27 10:18:28  jims
 * Fix comment
 *
 * Revision 7.23  1996/02/19 12:34:12  jims
 * Port to Stdlib 1.2
 *
 * Revision 7.22  1996/02/18 03:14:31  kyle
 * Millenium added support for compilers without ability to adjust access.
 *
 * Revision 7.21  1996/02/18 01:47:40  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.20  1996/02/14 23:00:13  jims
 * Add ctor from const container_type&
 *
 * Revision 7.19  1996/02/07 02:42:30  jims
 * Standard library extensions now in directory rw/stdex
 *
 * Revision 7.18  1996/02/01 21:50:02  jims
 * Separate out hash-only set operations into separate header
 *
 * Revision 7.17  1996/01/25 01:51:32  kevinj
 * Improved ETP macros.
 *
 * Revision 7.16  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.15  1996/01/16 22:37:38  kevinj
 * Simple External Template Persistence (ETP).
 *
 * Revision 7.14  1996/01/15 19:56:58  kevinj
 * Patched for OTIS0017 compile.
 *
 * Revision 7.13  1996/01/08 18:05:59  kevinj
 * *** empty log message ***
 *
 * Revision 7.12  1996/01/04 00:51:24  pearson
 * Change isSubset to isSubsetOf
 *
 * Revision 7.11  1995/12/20 23:19:20  kevinj
 * External template persistence using RW*Persistor.
 *
 * Revision 7.10  1995/12/15 00:27:57  pearson
 * Remove isUnequal and rename isEqual to isEquivalent
 * Move isEquivalent, isSubset and isProperSubset to the base classAdd
 * access adjustment due to private inheritance
 *
 * Revision 7.9  1995/12/14 21:44:31  pearson
 * Remove intersectWith and streamline isSubset
 *
 * Revision 7.8  1995/12/14 19:06:48  pearson
 * Make interface match non-STL based containers
 *
 * Revision 7.7  1995/12/13 00:22:58  jims
 * Remove member functions saveOn and restoreFrom (now global)
 *
 * Revision 7.6  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.5  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.4  1995/11/17 08:15:43  jims
 * Merge otis-et and otis-stl templates
 *
 * Revision 7.9  1995/10/20 06:46:20  jims
 * Port to msvc 4.0: don't use access adjustment for typedefs
 *
 * Revision 7.8  1995/10/11  22:36:45  griswolf
 * Scopus #2160: pass more constructor args through to implementation.
 *
 * Revision 7.7  1995/09/25  11:12:34  jims
 * Add #include for ep_hash.cc
 * Add NO_CONST_INST version of T**,T** ctor
 *
 * Revision 7.6  1995/09/21  00:23:55  griswolf
 * Provide correct set-method calls.
 * Add default size to constructor.
 *
 * Revision 7.5  1995/09/18  18:42:20  griswolf
 * Add capacity, fillRatio, resize methods.
 * Add difference, intersection, symmetric difference and Union methods.
 *
 * Revision 7.4  1995/09/12  19:41:49  griswolf
 * Change to #include renamed rw/rwstl/ header file.
 *
 * Revision 7.3  1995/09/05  21:51:20  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.2  1995/08/31  12:03:51  jims
 * next rev
 *
 * Revision 7.1  1995/08/31  06:45:31  jims
 * Move to revision 7.1
 * 
 ****************************************************************************/

#include "rw/compiler.h"

#ifdef RW_NO_STL

#include "rw/xphset.h"
#define RWDefHArgs(T)

#else

#ifndef RWDefHArgs
#  define RWDefHArgs(T) ,RWTHasher<T>,equal_to<T>
#endif

// STL-based implementation:


#include "rw/stdex/hashset.h"
#include "rw/ep_hash.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class T,class H,class EQ>
class RWTExport RWTPtrHashSet
    : private RW_PHAssoc<
      rw_hashset<T*, rw_deref_hash<H,T>,rw_deref_compare<EQ,T> >,
                 RWTPtrHashSet<T,H,EQ>, T>
{
public:
// typedefs
  typedef rw_deref_compare<EQ,T>                      container_eq;
  typedef rw_deref_hash<H,T>                          container_hash;
  typedef rw_hashset<T*,container_hash, container_eq> container_type;
  typedef RWTPtrHashSet<T,H,EQ>                       this_type;
  typedef RW_PHAssoc<container_type, this_type, T>    base_type;
  typedef base_type base1_type;
  
# ifndef RW_BROKEN_ACCESS_ADJUST
   typedef base1_type base2_type;
   typedef base1_type base3_type;
   typedef base1_type base4_type;
# else  
   typedef RW_PAssoc<container_type, this_type, T>     base2_type;
   typedef RW_PCntnr<container_type, this_type, T>     base3_type;
   typedef RW_PBase<container_type, this_type>         base4_type;
# endif

  typedef container_type::size_type size_type;
  typedef container_type::difference_type difference_type;
  
  typedef base_type::iterator iterator;
  typedef base_type::const_iterator const_iterator;
  typedef base_type::value_type value_type;
  typedef base_type::const_value const_value;
  typedef base_type::reference reference;
  typedef base_type::const_reference const_reference;

// STL-level access
  const container_type& std() const { return impl_; }
  container_type& std() { return impl_; }

// inherited interface components
  base4_type::begin; 
  base4_type::isEmpty;
  base4_type::end;
  base4_type::entries;
  base2_type::clear;
  base2_type::clearAndDestroy;
  base2_type::contains;
  base2_type::find;
  base2_type::occurrencesOf;
  base2_type::remove;
  base2_type::removeAll;
  base2_type::isSubsetOf;
  base2_type::isProperSubsetOf;
  base2_type::isEquivalent;

  RWTPtrHashSet<T,H,EQ> &
    Union(const RWTPtrHashSet<T,H,EQ>& rhs)
    { base_type::UnionHash(rhs); return *this; }

  RWTPtrHashSet<T,H,EQ> &
    intersection(const RWTPtrHashSet<T,H,EQ>& rhs)
    { base_type::intersectionHash(rhs); return *this; }

  RWTPtrHashSet<T,H,EQ> &
    difference(const RWTPtrHashSet<T,H,EQ>& rhs)
    { base_type::differenceHash(rhs); return *this; }

  RWTPtrHashSet<T,H,EQ> &
    symmetricDifference(const RWTPtrHashSet<T,H,EQ>& rhs)
    { base_type::symmetricDifferenceHash(rhs); return *this; }

// overridden interface comoponents
  bool operator==(const this_type& rwset) const
    { return entries() == rwset.entries() && hashCollEqual(*this, rwset); }

// selective inheritance (hides other same-name components)
  void  apply(void (*fn)(const_value,void*),void *x) const
    { base_type::apply(fn,x); }

// additional interface components
  bool  insert(value_type datum)
    { return (impl_.insert(datum)).second; }

  size_type
    capacity() const { return impl_.capacity(); }

  float
    fillRatio() const { return impl_.fill_ratio(); }

  void
    resize(size_type newsz) { impl_.resize(newsz); }

// assignment operators
  this_type&   operator=(const this_type& rwset)
    { impl_ = rwset.impl_; return *this; }
  
// constructors
  RWTPtrHashSet(size_type sz=1024, const H& h = H(), const EQ& eq = EQ())
    : impl_(sz,rw_deref_hash<H,T>(h),rw_deref_compare<EQ,T>(eq)) { }

  RWTPtrHashSet(const this_type& rws)                  : impl_(rws.impl_) { }
  RWTPtrHashSet(const container_type& s)               : impl_(s) { }

  RWTPtrHashSet(T*const* first,T*const* last,
            size_type sz=1024, const H& h = H(), const EQ& eq = EQ())
    : impl_(first,last,
            sz,rw_deref_hash<H,T>(h),rw_deref_compare<EQ,T>(eq)) {}

  // Tools-6 compatible constructor:
  RWTPtrHashSet(const H& h, size_type sz = RWDEFAULT_CAPACITY)
    : impl_(sz,h,EQ()) { }

private:
  container_type impl_;
  friend class RW_PBase<container_type, this_type>;
};

RWDECLARE_PERSISTABLE_TEMPLATE_3(RWTPtrHashSet)

#ifdef RW_COMPILE_INSTANTIATE
# include "rw/tphset.cc"
#endif


/****************************************************************
 *                                                              *
 *      Declarations for RWTPtrHashSetIterator<T,H,EQ>          *
 *                                                              *
 ****************************************************************/

template <class T, class H, class EQ>
class RWTExport RWTPtrHashSetIterator
{ 

public:

  RWTPtrHashSetIterator(RWTPtrHashSet<T,H,EQ>& t)
          : table_(&t), head_(true), tail_(false) { }

  RWBoolean     operator++()    // Advance and test
  {
    RWASSERT(!(head_ == true  &&  tail_ == true)); 

    if (tail_)
      /* do nothing */ ;

    else if (head_) {
      iter_ = table_->begin();
      head_ = false;
    }
    else
      ++iter_;

    if (!tail_ && iter_ == table_->end())
      tail_ = true;

    RWASSERT(!(head_ == true  &&  tail_ == true)); 

    return !tail_;
  }

  T*    operator()()    { return (++(*this)) ? key() : rwnil; }
        
  RWTPtrHashSet<T,H,EQ>* container() const  { return table_; }

  T*    key() const   { return *iter_; }

  void  reset()       { head_ = true; tail_ = false; } 
  void  reset(RWTPtrHashSet<T,H,EQ>& t)
                              { head_ = true; tail_ = false; table_ = &t; } 
private:

  RWTPtrHashSet<T,H,EQ>*                table_;
  RWTPtrHashSet<T,H,EQ>::iterator       iter_;
  bool                                  head_;
  bool                                  tail_;

  // Disallow postfix increment.  Unless we hide it, some compilers will
  // substitute the prefix increment operator in its place.
  RWBoolean             operator++(int);
};

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* RW_NO_STL */

#endif /* __RWTPHSET_H__ */
