#ifndef __RWTPORDVEC_H__
#define __RWTPORDVEC_H__

/***************************************************************************
 *
 * tpordvec.h - RWTPtrOrderedVector<T>
 *   
 *     : pointer-based ordered vector wrapper for STL vector
 * 
 * $Id: tpordvec.h,v 7.19 1996/05/07 19:57:01 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: tpordvec.h,v $
 * Revision 7.19  1996/05/07 19:57:01  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.18  1996/02/19 12:34:12  jims
 * Port to Stdlib 1.2
 *
 * Revision 7.17  1996/02/18 03:08:25  kyle
 * Millenium added support for broken access ajustments and for compilers
 * needing extensions to access StdLib
 *
 * Revision 7.16  1996/02/18 01:47:47  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.15  1996/02/13 12:14:17  jims
 * Small mods to the interface
 *
 * Revision 7.14  1996/01/25 01:51:32  kevinj
 * Improved ETP macros.
 *
 * Revision 7.13  1996/01/24 21:07:41  jims
 * Simplify implementation of otis-stl ptr-based collections
 * by using container<T*> impls instead of container<void*>
 *
 * Revision 7.12  1996/01/23 07:40:23  jims
 * Change ctor parameters from T** to T*const*
 *
 * Revision 7.11  1996/01/13 22:07:32  kevinj
 * Simple External Template Persistence (ETP).
 *
 * Revision 7.10  1995/12/20 23:19:20  kevinj
 * External template persistence using RW*Persistor.
 *
 * Revision 7.9  1995/12/14 18:01:21  jims
 * saveOn, restoreFrom now global instead of member functions
 *
 * Revision 7.8  1995/12/13 00:22:58  jims
 * Remove member functions saveOn and restoreFrom (now global)
 *
 * Revision 7.7  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.6  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.5  1995/11/13 12:47:12  jims
 * New stl-based collections now one with Tools-6 templates
 *
 * Revision 7.12  1995/10/20 06:46:20  jims
 * Port to msvc 4.0: don't use access adjustment for typedefs
 *
 * Revision 7.11  1995/09/25  11:09:35  jims
 * Add NO_CONST_INST version of T**,T** ctor
 *
 * Revision 7.10  1995/09/15  21:14:32  griswolf
 * Remove un-needed destructor, leaving it commented as documentation.
 * Add method sort().
 *
 * Revision 7.9  1995/09/05  21:51:45  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.8  1995/08/18  17:47:26  kevinj
 * Added RW_DEFINE_PERSISTABLE_COLLECTION.
 *
 * Revision 7.7  1995/08/15  00:20:48  kevinj
 * Added restoreFrom and saveOn.
 *
 * Revision 7.6  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/compiler.h"

#ifdef RW_NO_STL
#  include "rw/xpordvec.h"
#else
// STL-based implementation:

#ifndef RW_HEADER_WITH_EXTENSION
# include <vector>
#else
# include <vector.hpp>
#endif  /* RW_HEADER_WITH_EXTENSION */

#include "rw/ep_seq.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class T>
class RWTExport RWTPtrOrderedVector
 : private RW_PSeq<vector<T*,allocator>, RWTPtrOrderedVector<T>, T>
{
public:
// typedefs
  typedef vector<T*,allocator>                container_type;
  typedef RWTPtrOrderedVector<T>              this_type;
  typedef RW_PSeq<container_type,
                  RWTPtrOrderedVector<T>, T>  base1_type;

  typedef container_type::iterator          iterator;
  typedef container_type::const_iterator    const_iterator;
  typedef container_type::size_type         size_type;
  typedef container_type::difference_type   difference_type;
  typedef container_type::value_type        value_type;
  typedef container_type::reference         reference;
  typedef container_type::const_reference   const_reference;

// STL-level access
  const container_type&  std() const { return impl_; }
  container_type&        std()       { return impl_; }

#ifndef RW_BROKEN_ACCESS_ADJUST
  typedef base1_type base2_type;
  typedef base1_type base3_type;
#else
  //  inherited interface components cannot always follow base_type
  typedef RW_PCntnr<container_type,this_type,T> base2_type;
  typedef RW_PBase<container_type,this_type> base3_type;
#endif  
  
  base1_type::append;
  base2_type::apply;
  base3_type::at;
  base3_type::begin;
  base1_type::clear;
  base1_type::clearAndDestroy;
  base2_type::contains;
  base3_type::end;
  base3_type::entries;
  base2_type::find;
  base3_type::first;
  base2_type::index;
  base1_type::insert;
  base1_type::insertAt;
  base3_type::isEmpty;
  base1_type::minElement;
  base1_type::maxElement;
  base3_type::npos;
  base2_type::occurrencesOf;
  base3_type::operator();
  base2_type::operator<;
  base2_type::operator==;
  base3_type::operator[];
  base1_type::prepend;
  base1_type::remove;
  base1_type::removeAll;
  base1_type::removeAt;
  base1_type::removeFirst;
  base1_type::replaceAll;

// additional interface components
  T* const*     data() const       { return &(T* const &)(*(begin())); }
  T*&           last()             { return impl_.back(); }     
  T*const&      last() const       { return impl_.back(); }     
  size_type     length() const     { return impl_.size(); }
  void          resize(size_t n)   { impl_.reserve(n); }
  T*            removeLast()       { T* ret = last();
                                     impl_.pop_back();
                                     return ret;
                                    }
  void          sort()
    { ::sort(begin(),end(),rw_deref_compare<less<T>,T>()); }

// assignment operators
  RWTPtrOrderedVector<T>&   operator=(const RWTPtrOrderedVector<T>& ovec)
    { impl_ = ovec.impl_; return *this; }
  RWTPtrOrderedVector<T>&   operator=(const container_type& stdv)
    { impl_ = stdv; return *this; }

// constructors
  RWTPtrOrderedVector()                                : impl_() {}
  RWTPtrOrderedVector(const container_type& lst)       : impl_(lst) {}
  RWTPtrOrderedVector(const this_type& lst)            : impl_(lst.impl_) {}
  RWTPtrOrderedVector(size_type n, T* p)               : impl_(n, p) {}
  RWTPtrOrderedVector(T*const* first, T*const* last)   : impl_(first,last) {}

  // For Tools-6 compatibility (not particularly efficient):
  RWTPtrOrderedVector(size_type n) { resize(n); }

//  ~RWTPtrOrderedVector(); // handled by compiler-generated destructor

private:
  container_type impl_;         

  friend class RW_PBase<container_type, this_type>;
};

RWDECLARE_PERSISTABLE_TEMPLATE(RWTPtrOrderedVector)

#ifdef RW_COMPILE_INSTANTIATE
# include "rw/tpordvec.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* RW_NO_STL */

#endif /* __RWTPORDVEC_H__ */
