#ifndef __RWTVDEQUE_H__
#define __RWTVDEQUE_H__

/***************************************************************************
 *
 * tvdeque.h - RWTValDeque<T>
 *   
 *     : value-based double-ended queue wrapper for STL deque 
 * 
 * $Id: tvdeque.h,v 7.18 1996/05/07 19:24:10 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: tvdeque.h,v $
 * Revision 7.18  1996/05/07 19:24:10  kyle
 * Added export pragma for Mac code fragments
 * Changed RW_NO_STL from #endif to #else, #endif
 *
 * Revision 7.17  1996/03/20 09:13:06  jims
 * Remove bounds check on last() for consistency
 * with other sequenced-based collections
 *
 * Revision 7.16  1996/02/19 12:34:12  jims
 * Port to Stdlib 1.2
 *
 * Revision 7.15  1996/02/18 01:48:52  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.14  1996/02/16 21:45:24  kyle
 * fixed StdLib header support
 *
 * Revision 7.12  1996/01/30 21:04:24  jims
 * Take removeLast() out-of-line
 *
 * Revision 7.11  1996/01/25 01:51:32  kevinj
 * Improved ETP macros.
 *
 * Revision 7.10  1996/01/11 01:33:10  kevinj
 * RWFile external template persistence.
 *
 * Revision 7.9  1996/01/10 21:09:42  kevinj
 * RWDEFINE_PERSISTABLE_CONTAINER and RWDECLARE_PERSISTABLE_CONTAINER.
 *
 * Revision 7.8  1996/01/09 22:26:03  kevinj
 * Simpler external template persistence.
 *
 * Revision 7.7  1995/12/20 23:19:20  kevinj
 * External template persistence using RW*Persistor.
 *
 * Revision 7.6  1995/12/14 18:01:21  jims
 * saveOn, restoreFrom now global instead of member functions
 *
 * Revision 7.5  1995/12/13 00:22:58  jims
 * Remove member functions saveOn and restoreFrom (now global)
 *
 * Revision 7.4  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.3  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.2  1995/12/01 18:09:30  jims
 * Rename to 6.1 style name
 *
 * Revision 7.1  1995/11/29 20:19:34  jims
 * Move to revision 7.1
 *
 * Revision 7.15  1995/10/20  06:57:20  jims
 * include compiler.h before checking for RW_NO_STL
 *
 * Revision 7.14  1995/09/15  21:14:32  griswolf
 * Remove un-needed destructor, leaving it commented as documentation.
 *
 * Revision 7.13  1995/09/05  21:53:41  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.12  1995/08/18  16:41:39  kevinj
 * Removed edefs.h
 *
 * Revision 7.11  1995/08/17  22:55:28  kevinj
 * Added RW_DEFINE_PERSISTABLE_COLLECTION.
 *
 * Revision 7.10  1995/08/02  00:04:36  jims
 * Qualify include file names with "rw/"
 *
 * Revision 7.9  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 *
 ****************************************************************************/

#include "rw/compiler.h"

#ifdef RW_NO_STL
# error RWTValDeque<T> not available if RW_NO_STL is defined
#else // djk: was endif

#ifndef RW_HEADER_WITH_EXTENSION     // djk
# include <deque>
#else
# include <deque.hpp>
#endif  /* RW_HEADER_WITH_EXTENSION */

#include "rw/ev_seq.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class T>
class RWTExport RWTValDeque : private  RW_VSeq<deque<T,allocator>, RWTValDeque<T> >
{
public:
// typedefs
  typedef deque<T,allocator> container_type;
  typedef RWTValDeque<T> this_type;
  typedef RW_VSeq<container_type,this_type> base1_type;

  typedef container_type::iterator iterator;
  typedef container_type::const_iterator const_iterator;
  typedef container_type::size_type size_type;
  typedef container_type::value_type value_type;
  typedef container_type::reference reference;
  typedef container_type::const_reference const_reference;

#ifndef RW_BROKEN_ACCESS_ADJUST
  typedef base1_type base2_type;
  typedef base1_type base3_type;
#else
  typedef RW_VCntnr<container_type, this_type> base2_type;   // djk
  typedef RW_VBase<container_type, this_type> base3_type;   // djk
#endif

// STL-level access
  container_type&  std()             { return impl_; }
  const container_type&  std() const { return impl_; }

// inherited interface components
  base1_type::append;
  base3_type::apply;
  base3_type::at;
  base3_type::begin;
  base1_type::clear;
  base3_type::contains;
  base3_type::end;
  base3_type::entries;
  base2_type::find;
  base3_type::first;
  base3_type::index;
  base1_type::insert;
  base1_type::insertAt;
  base3_type::isEmpty;
  base2_type::minElement;
  base2_type::maxElement;
  base3_type::npos;
  base3_type::occurrencesOf;
  base3_type::operator();
  base3_type::operator<;
  base3_type::operator==;
  base3_type::operator[];
  base1_type::prepend;
  base1_type::remove;
  base1_type::removeAll;
  base1_type::removeAt;
  base1_type::removeFirst;
  base1_type::replaceAll;
  
// additional interface components
  T&          last()                 { return std().back(); }
  const T&    last() const           { return std().back(); }
  void        pushBack(const T& t)   { std().push_back(t); }
  void        pushFront(const T& t)  { std().push_front(t); }
  T           popBack()              { return removeLast(); }
  T           popFront()             { return removeFirst(); }
  void        sort()                 { ::sort(begin(),end()); }
  value_type  removeLast();
  
// assignment operators
  RWTValDeque<T>&   operator=(const RWTValDeque<T>& rwset)
    { impl_ = rwset.impl_; return *this; }
  RWTValDeque<T>&   operator=(const container_type& stdset)
    { impl_ = stdset; return *this; }
  
// constructors
  RWTValDeque()                                         : impl_() { }
  RWTValDeque(const container_type& lst)                : impl_(lst) { }
  RWTValDeque(const RWTValDeque<T>& lst)                : impl_(lst.impl_) { }
  RWTValDeque(container_type::size_type n, const T& val = T())
                                                        : impl_(n,val) { }
  RWTValDeque(const T* first, const T* last)            : impl_(first,last) {}
//  ~RWTValDeque(); // handled by compiler-generated destructor

private:
  container_type impl_;
  friend class RW_VBase<container_type,this_type>;
};

RWDECLARE_PERSISTABLE_TEMPLATE(RWTValDeque)

#ifdef RW_COMPILE_INSTANTIATE
# include "rw/tvdeque.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* RW_NO_STL */

#endif /* __RWTVDEQUE_H__ */
