#ifndef __RWTVSLIST_H__
#define __RWTVSLIST_H__

/***************************************************************************
 *
 * tvslist.h - RWTValSlist<T>
 *   
 *     : value-based singly-linked list wrapper for STL list
 * 
 * $Id: tvslist.h,v 7.18 1996/05/07 18:32:08 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 * 
 * $Log: tvslist.h,v $
 * Revision 7.18  1996/05/07 18:32:08  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.17  1996/03/14 20:31:09  kyle
 * Changed rw_slist<T> impl_ to use typedef.
 *
 * Revision 7.16  1996/02/18 01:50:19  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.15  1996/02/16 20:37:03  kyle
 * Millenium added support for compilers where inherited interface components
 * cannot always follow base_type
 *
 * Revision 7.14  1996/02/07 02:42:30  jims
 * Standard library extensions now in directory rw/stdex
 *
 * Revision 7.13  1996/01/25 01:51:32  kevinj
 * Improved ETP macros.
 *
 * Revision 7.12  1996/01/13 23:26:30  kevinj
 * Simple External Template Persistence (ETP).
 *
 * Revision 7.11  1995/12/20 23:19:20  kevinj
 * External template persistence using RW*Persistor.
 *
 * Revision 7.10  1995/12/20 20:25:51  pearson
 * Add include for implementation for compilers which do not automatically
 * find .cc files
 *
 * Revision 7.9  1995/12/14 18:01:21  jims
 * saveOn, restoreFrom now global instead of member functions
 *
 * Revision 7.8  1995/12/13 00:22:58  jims
 * Remove member functions saveOn and restoreFrom (now global)
 *
 * Revision 7.7  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.6  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.5  1995/11/13 17:55:03  jims
 * Add missing semicolon, remove extraneous comma,
 * ( RWTValSlistIterator::reset )
 *
 * Revision 7.4  1995/11/13 12:47:12  jims
 * New stl-based collections now one with Tools-6 templates
 *
 * Revision 7.14  1995/09/15 21:14:32  griswolf
 * Remove un-needed destructor, leaving it commented as documentation.
 *
 * Revision 7.13  1995/09/12  19:41:49  griswolf
 * Change to #include renamed rw/rwstl/ header file.
 *
 * Revision 7.12  1995/09/05  21:56:44  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.11  1995/08/18  22:35:06  kevinj
 * Added RW_DEFINE_PERSISTABLE_COLLECTION.
 *
 * Revision 7.10  1995/08/15  17:19:13  kevinj
 * Added restoreFrom and saveOn but was unable to test them.
 *
 * Revision 7.9  1995/08/02  00:04:36  jims
 * Qualify include file names with "rw/"
 *
 * Revision 7.8  1995/07/31  23:18:29  jims
 * Change name of ValSlist to rw_slist
 *
 * Revision 7.7  1995/07/29  00:20:39  jims
 * Fix #include for vslist.h
 *
 * Revision 7.6  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 * 
 ****************************************************************************/

#include "rw/compiler.h"

#ifdef RW_NO_STL
#  include "rw/xvslist.h"
#else
// STL-based implementation:

#include "rw/stdex/slist.h"       // Rogue Wave STL-type singly linked list
#include "rw/ev_seq.h"

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class T>
class RWTExport RWTValSlist : private RW_VSeq<rw_slist<T>,RWTValSlist<T> >
{
public:
// typedefs
  typedef rw_slist<T> container_type;
  typedef RWTValSlist<T> this_type;
  typedef RW_VSeq<container_type,this_type> base1_type;
  
#  ifndef RW_BROKEN_ACCESS_ADJUST
     typedef base1_type base2_type;
     typedef base1_type base3_type;
#  else
     typedef RW_VCntnr<container_type,this_type> base2_type;
     typedef RW_VBase<container_type,this_type>  base3_type;
#  endif  /* RW_BROKEN_ACCESS_ADJUST */
  
  typedef container_type::iterator iterator;
  typedef container_type::const_iterator const_iterator;
  typedef container_type::size_type size_type;
  typedef container_type::value_type value_type;
  typedef container_type::reference reference;
  typedef container_type::const_reference const_reference;

// STL-level access
  container_type&  std()             { return impl_; }
  const container_type&  std() const { return impl_; }

// inherited interface components
  base1_type::append;
  base3_type::apply;
  base3_type::at;
  base1_type::begin;
  base1_type::clear;
  base3_type::contains;
  base1_type::end;
  base3_type::entries;
  base2_type::find;
  base1_type::first;
  base1_type::get;
  base3_type::index;
  base1_type::insert;
  base1_type::insertAt;
  base3_type::isEmpty;
  base1_type::minElement;
  base1_type::maxElement;
  base1_type::npos;
  base3_type::occurrencesOf;
  base1_type::operator();
  base1_type::operator<;
  base1_type::operator==;
  base3_type::operator[];
  base1_type::prepend;
  base1_type::remove;
  base1_type::removeAll;
  base1_type::removeAt;
  base1_type::removeFirst;
  base1_type::replaceAll;
  
// overridden interface components
  void sort() { std().sort(); }
  
// additional interface components
  T     last()       { return std().back(); }
  T     removeLast() { T ret = last(); removeAt(entries()-1); return ret; }
  
// assignment operators
  RWTValSlist<T>&   operator=(const RWTValSlist<T>& rwset)
    { impl_ = rwset.impl_; return *this; }
  RWTValSlist<T>&   operator=(const rw_slist<T>& stdset)
    { impl_ = stdset; return *this; }
  
// constructors
  RWTValSlist()                                : impl_() {}
  RWTValSlist(const rw_slist<T>& lst)          : impl_(lst) {}
  RWTValSlist(const RWTValSlist<T>& lst)       : impl_(lst.impl_) {}
  RWTValSlist(size_type n, const T& t = T())   : impl_(n,t) {}
  RWTValSlist(const T* first, const T*last)    : impl_(first,last) {}
//  ~RWTValSlist(); // handled by auto-generated destructor

private:
  container_type impl_;         // djk: use typedef
  friend class RW_VBase<container_type,this_type>;

};

RWDECLARE_PERSISTABLE_TEMPLATE(RWTValSlist)

/****************************************************************
 *                                                              *
 *      Declarations for RWTValSlistIterator<T>                 *
 *                                                              *
 ****************************************************************/

template <class T> class RWTExport RWTValSlistIterator
{
private:
  RWTValSlist<T>*           list_;
  RWTValSlist<T>::iterator  iter_;
  bool                      head_;  // simulate head sentinel
  bool                      tail_;  // simulate tail sentinel
/*
 * Invariants:
 *   head_ and tail_ can never both be true
 *   if head_ and tail_ are both false then iter_ is valid
 *     (unless invalidated from outside this class)
 */

public:

  RWTValSlistIterator(RWTValSlist<T>& s)
          : list_(&s), head_(true), tail_(false) { }

  // Operators:
  RWBoolean     operator++();
  RWBoolean     operator+=(size_t n);
  RWBoolean     operator()()        { return ++(*this); }

  // Methods
  RWTValSlist<T>* container() const { return list_; }

  RWBoolean     findNext(const T& a);
  RWBoolean     findNext(RWBoolean (*testFun)(const T&, void*), void*);

  void          insertAfterPoint(const T& a);

  T             key() const   { return *iter_; }

  RWBoolean     remove();       // Remove item at cursor

  RWBoolean     removeNext(const T&);
  RWBoolean     removeNext(RWBoolean (*testFun)(const T&, void*), void*);

  void          reset()       { head_ = true; tail_ = false; } 
  void          reset(RWTValSlist<T>& s)
    { head_ = true; tail_ = false; list_ = &s; } 

private:

  // Disallow postfix increment.  Unless we hide it, some compilers will
  // substitute the prefix increment operator in its place.
  RWBoolean             operator++(int);
};

#ifdef RW_COMPILE_INSTANTIATE
#include "rw/tvslist.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* RW_NO_STL */

#endif /* __RWTVSLIST_H__ */
