#ifndef __RWTVSRTVEC_H__
#define __RWTVSRTVEC_H__

/***************************************************************************
 *
 * tvsrtvec.h - RWTValSortedVector<T,Compare>
 *
 *     : value-based sorted vector wrapper for STL vector
 *
 * $Id: tvsrtvec.h,v 7.21 1996/05/07 18:31:30 kyle Exp $
 *
 * (c) Copyright 1989 - 1996 Rogue Wave Software, Inc.
 * ALL RIGHTS RESERVED
 *
 * The software and information contained herein are proprietary to, and
 * comprise valuable trade secrets of, Rogue Wave Software, Inc., which
 * intends to preserve as trade secrets such software and information.
 * This software is furnished pursuant to a written license agreement and
 * may be used, copied, transmitted, and stored only in accordance with
 * the terms of such license and with the inclusion of the above copyright
 * notice.  This software and information or any other copies thereof may
 * not be provided or otherwise made available to any other person.
 *
 * Notwithstanding any other lease or license that may pertain to, or
 * accompany the delivery of, this computer software and information, the
 * rights of the Government regarding its use, reproduction and disclosure
 * are as set forth in Section 52.227-19 of the FARS Computer
 * Software-Restricted Rights clause.
 * 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013.
 * Contractor/Manufacturer is Rogue Wave Software, Inc.,
 * P.O. Box 2328, Corvallis, Oregon 97339.
 *
 * This computer software and information is distributed with "restricted
 * rights."  Use, duplication or disclosure is subject to restrictions as
 * set forth in NASA FAR SUP 18-52.227-79 (April 1985) "Commercial
 * Computer Software-Restricted Rights (April 1985)."  If the Clause at
 * 18-52.227-74 "Rights in Data General" is specified in the contract,
 * then the "Alternate III" clause applies.
 *
 ****************************************************************************
 *
 * $Log: tvsrtvec.h,v $
 * Revision 7.21  1996/05/07 18:31:30  kyle
 * Added export pragma for Mac code fragments
 *
 * Revision 7.20  1996/04/29 19:12:43  groening
 * needed to include compiler.h prior to making any references to
 * RW_NO_STL.
 *
 * Revision 7.19  96/03/21  23:50:15  23:50:15  pearson (Thomas Pearson)
 * #2904 - Place #include "rw/ev_srtsq.h" before #include <vector>
 * 
 * Revision 7.18  1996/03/14 16:49:15  griswolf
 * Avoid possible "redefinition of RWDefCargs" warnings.
 *
 * Revision 7.17  1996/02/19 12:34:12  jims
 * Port to Stdlib 1.2
 *
 * Revision 7.16  1996/02/18 01:50:25  griswolf
 * Replace tabs with spaces, per Rogue Wave standard.
 *
 * Revision 7.15  1996/02/16 20:40:33  kyle
 * fixed support for base1_type instead of base_type
 *
 * Revision 7.14  1996/02/16 20:11:54  kyle
 * Millemium adds support when inherited interface components cannot
 * always follow base_type and support for StdLib compilers which
 * need a .hpp extension instead of without extensions
 *
 * Revision 7.13  1996/02/07 06:04:13  jims
 * Remove devnotes (#2555)
 *
 * Revision 7.12  1996/01/30 08:40:56  jims
 * Use local iterator type instead of container_type::iterator in ctor
 *
 * Revision 7.11  1996/01/25 01:51:32  kevinj
 * Improved ETP macros.
 *
 * Revision 7.10  1996/01/14 00:10:18  kevinj
 * Simple External Template Persistence (ETP).
 *
 * Revision 7.9  1995/12/20 23:19:20  kevinj
 * External template persistence using RW*Persistor.
 *
 * Revision 7.8  1995/12/14 18:01:21  jims
 * saveOn, restoreFrom now global instead of member functions
 *
 * Revision 7.7  1995/12/13 00:22:58  jims
 * Remove member functions saveOn and restoreFrom (now global)
 *
 * Revision 7.6  1995/12/12 00:03:06  jims
 * Add RWTExport
 *
 * Revision 7.5  1995/12/11 22:30:41  jims
 * Remove 'Sca' from interior class names
 * Add 'rw_' to global helper classes
 * Change 'muterator' to 'filterator'
 *
 * Revision 7.4  1995/11/17 08:15:43  jims
 * Merge otis-et and otis-stl templates
 *
 * Revision 7.12  1995/09/18 17:42:07  jims
 * Remove prepend mf
 *
 * Revision 7.11  1995/09/15  21:14:32  griswolf
 * Remove un-needed destructor, leaving it commented as documentation.
 *
 * Revision 7.10  1995/09/05  21:57:03  jims
 * Change tlyrs.h to tlyrs.str
 *
 * Revision 7.9  1995/08/18  18:01:42  kevinj
 * Added RW_DEFINE_PERSISTABLE_COLLECTION.
 * ,
 *
 * Revision 7.8  1995/08/15  17:40:07  kevinj
 * Added restoreFrom and saveOn.
 *
 * Revision 7.7  1995/08/02  00:04:36  jims
 * Qualify include file names with "rw/"
 *
 * Revision 7.6  1995/07/27  22:02:33  jims
 * Add header with file name, short description, $Id and $Log
 *
 ****************************************************************************/

#include "rw/compiler.h"

#ifdef RW_NO_STL
#  include "rw/xvsrtvec.h"
#  undef  RWDefCArgs
#  define RWDefCArgs(T)
#else
#  ifndef RWDefCArgs
#    define RWDefCArgs(T) ,less<T>
#  endif

// STL-based implementation:
                         
#include "rw/ev_srtsq.h" // RW_VSrtSeq module (#2904-before #include<vector>)

#ifndef RW_HEADER_WITH_EXTENSION 
# include <vector>
#else
# include <vector.hpp>
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export on
#endif

template <class T, class C> // hold type and comparison object
class RWTExport RWTValSortedVector
         : private RW_VSrtSeq<vector<T,allocator>,RWTValSortedVector<T,C>, C>
{
public:
// typedefs
  typedef vector<T,allocator> container_type;
  typedef RWTValSortedVector<T,C> this_type;
  typedef RW_VSrtSeq<container_type,this_type,C> base1_type;
  typedef container_type::const_iterator iterator;
  typedef container_type::const_iterator const_iterator;
  typedef container_type::size_type size_type;
  typedef container_type::value_type value_type;
  typedef container_type::const_reference reference;
  typedef container_type::const_reference const_reference;

// STL-level access (const only)
  const container_type&  std() const { return impl_; }
  container_type&  std() { return impl_; }

public:  
// inherited interface components

#ifndef RW_BROKEN_ACCESS_ADJUST
  typedef base1_type base2_type;
  typedef base1_type base3_type;
#else
  typedef RW_VSeq<container_type,this_type> base2_type;
  typedef RW_VBase<container_type,this_type> base3_type;
#endif  /* RW_BROKEN_ACCESS_ADJUST */
  
  base3_type::begin;
  base2_type::clear;
  base1_type::contains;
  base3_type::end;
  base3_type::entries; 
  base1_type::find;
  base3_type::first;
  base3_type::index;
  base1_type::insert;
  base3_type::isEmpty;
  base1_type::isSorted;
  base3_type::npos;
  base1_type::occurrencesOf;
  base3_type::operator<;
  base3_type::operator==;
  base1_type::remove;
  base1_type::removeAll;
  base2_type::removeAt;
  base2_type::removeFirst;

// Allowing use as lvalue for the sake of Tools 6.1 compatiblity:
  base3_type::operator[];
  base3_type::operator();
  base3_type::at;

// selective inheritance (hides other same-name components)
  void              apply(void (*fn)(const_reference,void*),void *x) const
    { base1_type::apply(fn,x); }

// additional interface components
  const T*          data() const      { return &(const T&)(*begin()); }
  const_reference   last() const      { return std().back(); }
  size_type         merge(const this_type& coll)
    {
      size_type ret = coll.entries();
      if (ret) {
        size_type n = entries() + ret;
        container_type v(n);
        ::merge(begin(), end(), coll.begin(), coll.end(), v.begin(), C());
        impl_ = v;
      }
      return ret;
    }

  value_type        removeLast()
    { T ret = last(); impl_.pop_back(); return ret; }
  size_type         length() const    { return std().size(); }
  void              resize(size_t n)  { impl_.reserve(n); }


// constructors
  RWTValSortedVector() { }
  RWTValSortedVector(const RWTValSortedVector<T,C>& t) : impl_(t.impl_) { }
  RWTValSortedVector(const container_type& t) {
    resize(t.size()); // purely for efficiency
    insert(t);
  }
  RWTValSortedVector(size_type n, const T& val) : impl_(n,val) { }
  RWTValSortedVector(const T* first, const T* last);

  // For Tools-6 compatibility (not particularly efficient):
  RWTValSortedVector(size_type n) { resize(n); }


// ~RWTValSortedVector(); // handled by auto-generated destructor

private:
  container_type impl_;
  friend class RW_VBase<container_type,this_type>;
};

RWDECLARE_PERSISTABLE_TEMPLATE_2(RWTValSortedVector)

#ifdef RW_COMPILE_INSTANTIATE
# include "rw/tvsrtvec.cc"
#endif

#ifdef RW_PRAGMA_EXPORT
#pragma export off
#endif

#endif /* RW_NO_STL */

#endif /* __RWTVSRTVEC_H__ */
